# -*- coding: utf-8 -*-

#  -----------------
#  Copyright © 2023 ACK Cyfronet AGH, Poland.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#
#  This work was partially funded by DT-GEO Project.
#  -----------------

import numpy as np
from obspy import read
from functions import rs_func


def main(miniseed_file, Damping = 0.05):
    """
    Python application that reads the acceleration miniseed file and calculates the response spectra.
    Arguments:
        miniseed_file: path to input file of type 'MiniSEED in accleration unit'
        Damping: Damping factor. 0.05 means 5% damping ratio
    Returns:
        CSV files of the response spectra values.
    """
    Resp_types = ['SA','PSA','SV','PSV','SD']
    T = np.concatenate(
        (
            np.arange(0.05, 0.1, 0.005), #0.05s to 0.1s by 0.005s intervals
            np.arange(0.1, 0.5, 0.01), #0.1s to 0.5s by 0.01s intervals
            np.arange(0.5, 1, 0.02),
            np.arange(1, 5, 0.1),
            np.arange(5, 10.5, 0.5),
        )
    )
    stream_raw = read(miniseed_file)
    for resp in Resp_types:
        stream = stream_raw.copy() #start with fresh stream
        csv_header = ["Period"] #start of header for csv output file
        Sfin = np.zeros((len(stream), len(T)))
        for i, st in enumerate(stream):
            st.data = st.data * 100 #convert from m/s/s to cm/s/s
            dt = round(st.stats.delta, 3)  # Get dt sampling rate in seconds
            tr_data = st.data # extract time series
            Sfin[i] = rs_func(tr_data, dt, T, Damping, Resp_type=resp)
            net = st.stats.network
            stn = st.stats.station
            loc = st.stats.location
            chan = st.stats.channel
            nslc = net + '.' + stn + '.' + loc + '.' + chan
            csv_header.append(nslc + " " + resp)
        csv_array = np.vstack((T,Sfin))
        csv_array = csv_array.T          
        np.savetxt('spectra_'+resp+'.csv', csv_array, delimiter=",", header=",".join(csv_header))